REM
REM  Joypad emulation of a GameBoy console
REM  Part of the ZilogBoy project
REM  Copyright (C) Mateusz Viste 2010
REM

REM The joypad is controlled via the register at $FF00:
REM  Bit 5 - P15 out
REM  Bit 4 - P14 out
REM  Bit 3 - P13 in
REM  Bit 2 - P12 in
REM  Bit 1 - P11 in
REM  Bit 0 - P10 in
REM
REM  When one of the keys gets a transition from high to low, an interrupt is fired.

DIM SHARED AS UBYTE JoyOldReg = 0, JoyNewReg
DIM SHARED AS ULONGINT JoyNextCheck

SUB CheckJoypad()
  IF TotalCycles > JoyNextCheck THEN
    'SLEEP 1,1
    'JoyNextCheck += 20000   ' Check joypad state every 2.5 ms
    'JoyNextCheck += 20000   ' Check joypad state every 5 ms
    'JoyNextCheck += 40000   ' Check joypad state every 10 ms
    'JoyNewReg = &b00111111
    JoyNewReg = 0
    SELECT CASE IoRegisters(&hFF00) AND &b00110000   ' Get P14 and P15 state
      CASE &b100000  ' P14 selected (down/up/left/right)
        IF MULTIKEY(&h50) = -1 THEN JoyNewReg = JoyNewReg OR &b00001000 ' if DOWN -> Set P13
        IF MULTIKEY(&h48) = -1 THEN JoyNewReg = JoyNewReg OR &b00000100 ' if UP -> Set P12
        IF MULTIKEY(&h4B) = -1 THEN JoyNewReg = JoyNewReg OR &b00000010 ' if LEFT -> Set P11
        IF MULTIKEY(&h4D) = -1 THEN JoyNewReg = JoyNewReg OR &b00000001 ' if RIGHT -> Set P10
      CASE &b010000  ' P15 selected (Start/Select/B/A)
        IF MULTIKEY(&h1C) = -1 THEN JoyNewReg = JoyNewReg OR &b00001000 ' if ENTER -> Set P13
        IF MULTIKEY(&h0F) = -1 THEN JoyNewReg = JoyNewReg OR &b00000100 ' if TAB -> Set P12
        IF MULTIKEY(&h1D) = -1 THEN JoyNewReg = JoyNewReg OR &b00000010 ' if CTRL -> Set P11
        IF MULTIKEY(&h38) = -1 THEN JoyNewReg = JoyNewReg OR &b00000001 ' if ALT -> Set P10
    END SELECT
   IoRegisters(&hFF00) = IoRegisters(&hFF00) OR JoyNewReg    ' update the joypad register [FF00h]
    IF JoyOldReg <> JoyNewReg THEN                            ' If a key has been pressed/released
     JoyOldReg = JoyNewReg                                   ' update the JoyOldReg variable
     IoRegisters(&hFF0F) = IoRegisters(&hFF0F) OR &b00010000 ' Request a joypad INT
    END IF
  END IF
END SUB
